
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>
#include <gst/webrtc/webrtc_fwd.h>

G_BEGIN_DECLS

/* enumerations from "webrtc_fwd.h" */

GST_WEBRTC_API
GType gst_webrtc_dtls_transport_state_get_type (void);
#define GST_TYPE_WEBRTC_DTLS_TRANSPORT_STATE (gst_webrtc_dtls_transport_state_get_type())

GST_WEBRTC_API
GType gst_webrtc_ice_gathering_state_get_type (void);
#define GST_TYPE_WEBRTC_ICE_GATHERING_STATE (gst_webrtc_ice_gathering_state_get_type())

GST_WEBRTC_API
GType gst_webrtc_ice_connection_state_get_type (void);
#define GST_TYPE_WEBRTC_ICE_CONNECTION_STATE (gst_webrtc_ice_connection_state_get_type())

GST_WEBRTC_API
GType gst_webrtc_signaling_state_get_type (void);
#define GST_TYPE_WEBRTC_SIGNALING_STATE (gst_webrtc_signaling_state_get_type())

GST_WEBRTC_API
GType gst_webrtc_peer_connection_state_get_type (void);
#define GST_TYPE_WEBRTC_PEER_CONNECTION_STATE (gst_webrtc_peer_connection_state_get_type())

GST_WEBRTC_API
GType gst_webrtc_ice_role_get_type (void);
#define GST_TYPE_WEBRTC_ICE_ROLE (gst_webrtc_ice_role_get_type())

GST_WEBRTC_API
GType gst_webrtc_ice_component_get_type (void);
#define GST_TYPE_WEBRTC_ICE_COMPONENT (gst_webrtc_ice_component_get_type())

GST_WEBRTC_API
GType gst_webrtc_sdp_type_get_type (void);
#define GST_TYPE_WEBRTC_SDP_TYPE (gst_webrtc_sdp_type_get_type())

GST_WEBRTC_API
GType gst_webrtc_rtp_transceiver_direction_get_type (void);
#define GST_TYPE_WEBRTC_RTP_TRANSCEIVER_DIRECTION (gst_webrtc_rtp_transceiver_direction_get_type())

GST_WEBRTC_API
GType gst_webrtc_dtls_setup_get_type (void);
#define GST_TYPE_WEBRTC_DTLS_SETUP (gst_webrtc_dtls_setup_get_type())

GST_WEBRTC_API
GType gst_webrtc_stats_type_get_type (void);
#define GST_TYPE_WEBRTC_STATS_TYPE (gst_webrtc_stats_type_get_type())

GST_WEBRTC_API
GType gst_webrtc_fec_type_get_type (void);
#define GST_TYPE_WEBRTC_FEC_TYPE (gst_webrtc_fec_type_get_type())

GST_WEBRTC_API
GType gst_webrtc_sctp_transport_state_get_type (void);
#define GST_TYPE_WEBRTC_SCTP_TRANSPORT_STATE (gst_webrtc_sctp_transport_state_get_type())

GST_WEBRTC_API
GType gst_webrtc_priority_type_get_type (void);
#define GST_TYPE_WEBRTC_PRIORITY_TYPE (gst_webrtc_priority_type_get_type())

GST_WEBRTC_API
GType gst_webrtc_data_channel_state_get_type (void);
#define GST_TYPE_WEBRTC_DATA_CHANNEL_STATE (gst_webrtc_data_channel_state_get_type())

GST_WEBRTC_API
GType gst_webrtc_bundle_policy_get_type (void);
#define GST_TYPE_WEBRTC_BUNDLE_POLICY (gst_webrtc_bundle_policy_get_type())

GST_WEBRTC_API
GType gst_webrtc_ice_transport_policy_get_type (void);
#define GST_TYPE_WEBRTC_ICE_TRANSPORT_POLICY (gst_webrtc_ice_transport_policy_get_type())

GST_WEBRTC_API
GType gst_webrtc_kind_get_type (void);
#define GST_TYPE_WEBRTC_KIND (gst_webrtc_kind_get_type())

GST_WEBRTC_API
GType gst_webrtc_error_get_type (void);
#define GST_TYPE_WEBRTC_ERROR (gst_webrtc_error_get_type())

G_END_DECLS

/* Generated data ends here */

